@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,22 +18,24 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle;
 
+import com.puppycrawl.tools.checkstyle.api.Utils;
+
+import com.google.common.collect.Lists;
+import com.puppycrawl.tools.checkstyle.api.AuditListener;
+import com.puppycrawl.tools.checkstyle.api.Configuration;
+import com.puppycrawl.tools.checkstyle.api.SeverityLevel;
+import com.puppycrawl.tools.checkstyle.api.SeverityLevelCounter;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
-import java.util.ArrayList;
-import java.util.Iterator;
+import java.net.URL;
+import java.util.Hashtable;
 import java.util.List;
 import java.util.Properties;
-import java.util.Hashtable;
 import java.util.ResourceBundle;
-import java.net.URL;
-
-import com.puppycrawl.tools.checkstyle.api.AuditListener;
-import com.puppycrawl.tools.checkstyle.api.Configuration;
 import org.apache.tools.ant.AntClassLoader;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.DirectoryScanner;
@@ -50,8 +52,7 @@ import org.apache.tools.ant.types.Reference;
  * of the task for usage.
  * @author Oliver Burn
  */
-public class CheckStyleTask
-    extends Task
+public class CheckStyleTask extends Task
 {
     /** poor man's enum for an xml formatter */
     private static final String E_XML = "xml";
@@ -67,9 +68,6 @@ public class CheckStyleTask
     /** config file containing configuration */
     private String mConfigLocation;
 
-    /** contains package names */
-    private File mPackageNamesFile;
-
     /** whether to fail build on violations */
     private boolean mFailOnViolation = true;
 
@@ -77,17 +75,30 @@ public class CheckStyleTask
     private String mFailureProperty;
 
     /** contains the filesets to process */
-    private final List mFileSets = new ArrayList();
+    private final List<FileSet> mFileSets = Lists.newArrayList();
 
     /** contains the formatters to log to */
-    private final List mFormatters = new ArrayList();
+    private final List<Formatter> mFormatters = Lists.newArrayList();
 
     /** contains the Properties to override */
-    private final List mOverrideProps = new ArrayList();
+    private final List<Property> mOverrideProps = Lists.newArrayList();
 
     /** the name of the properties file */
     private File mPropertiesFile;
 
+    /** the maximum number of errors that are tolerated. */
+    private int mMaxErrors;
+
+    /** the maximum number of warnings that are tolerated. */
+    private int mMaxWarnings = Integer.MAX_VALUE;
+
+    /**
+     * whether to omit ignored modules - some modules may log above
+     * their severity depending on their configuration (e.g. WriteTag) so
+     * need to be included
+     */
+    private boolean mOmitIgnoredModules = true;
+
     ////////////////////////////////////////////////////////////////////////////
     // Setters for ANT specific attributes
     ////////////////////////////////////////////////////////////////////////////
@@ -109,6 +120,25 @@ public class CheckStyleTask
         mFailOnViolation = aFail;
     }
 
+    /**
+     * Sets the maximum number of errors allowed. Default is 0.
+     * @param aMaxErrors the maximum number of errors allowed.
+     */
+    public void setMaxErrors(int aMaxErrors)
+    {
+        mMaxErrors = aMaxErrors;
+    }
+
+    /**
+     * Sets the maximum number of warings allowed. Default is
+     * {@link Integer#MAX_VALUE}.
+     * @param aMaxWarnings the maximum number of warnings allowed.
+     */
+    public void setMaxWarnings(int aMaxWarnings)
+    {
+        mMaxWarnings = aMaxWarnings;
+    }
+
     /**
      * Adds a set of files (nested fileset attribute).
      * @param aFS the file set to add
@@ -193,17 +223,16 @@ public class CheckStyleTask
     private void setConfigLocation(String aLocation)
     {
         if (mConfigLocation != null) {
-            throw new BuildException(
-                "Attributes 'config' and 'configURL' "
-                + "must not be set at the same time");
+            throw new BuildException("Attributes 'config' and 'configURL' "
+                    + "must not be set at the same time");
         }
         mConfigLocation = aLocation;
     }
 
-    /** @param aFile the package names file to use */
-    public void setPackageNamesFile(File aFile)
+    /** @param aOmit whether to omit ignored modules */
+    public void setOmitIgnoredModules(boolean aOmit)
     {
-        mPackageNamesFile = aFile;
+        mOmitIgnoredModules = aOmit;
     }
 
     ////////////////////////////////////////////////////////////////////////////
@@ -224,29 +253,41 @@ public class CheckStyleTask
     // The doers
     ////////////////////////////////////////////////////////////////////////////
 
+    @Override
+    public void execute() throws BuildException
+    {
+        final long startTime = System.currentTimeMillis();
+
+        try {
+            realExecute();
+        }
+        finally {
+            final long endTime = System.currentTimeMillis();
+            log("Total execution took " + (endTime - startTime) + " ms.",
+                Project.MSG_VERBOSE);
+        }
+    }
+
     /**
-     * Actually checks the files specified. All errors are reported to
-     * System.out. Will fail if any errors occurred.
-     * @throws BuildException an error occurred
+     * Helper implementation to perform execution.
      */
-    public void execute()
-        throws BuildException
+    private void realExecute()
     {
         // output version info in debug mode
-        final ResourceBundle compilationProperties =
-            ResourceBundle.getBundle("checkstylecompilation");
-        final String version =
-            compilationProperties.getString("checkstyle.compile.version");
-        final String compileTimestamp =
-            compilationProperties.getString("checkstyle.compile.timestamp");
+        final ResourceBundle compilationProperties = ResourceBundle
+                .getBundle("checkstylecompilation");
+        final String version = compilationProperties
+                .getString("checkstyle.compile.version");
+        final String compileTimestamp = compilationProperties
+                .getString("checkstyle.compile.timestamp");
         log("checkstyle version " + version, Project.MSG_VERBOSE);
         log("compiled on " + compileTimestamp, Project.MSG_VERBOSE);
 
         // Check for no arguments
         if ((mFileName == null) && (mFileSets.size() == 0)) {
             throw new BuildException(
-                "Must specify atleast one of 'file' or nested 'fileset'.",
-                getLocation());
+                    "Must specify atleast one of 'file' or nested 'fileset'.",
+                    getLocation());
         }
 
         if (mConfigLocation == null) {
@@ -258,18 +299,42 @@ public class CheckStyleTask
         try {
             c = createChecker();
 
+            final SeverityLevelCounter warningCounter =
+                new SeverityLevelCounter(SeverityLevel.WARNING);
+            c.addListener(warningCounter);
+
             // Process the files
-            final File[] files = scanFileSets();
+            long startTime = System.currentTimeMillis();
+            final List<File> files = scanFileSets();
+            long endTime = System.currentTimeMillis();
+            log("To locate the files took " + (endTime - startTime) + " ms.",
+                Project.MSG_VERBOSE);
+
+            log("Running Checkstyle " + version + " on " + files.size()
+                    + " files", Project.MSG_INFO);
+            log("Using configuration " + mConfigLocation, Project.MSG_VERBOSE);
+
+            startTime = System.currentTimeMillis();
             final int numErrs = c.process(files);
+            endTime = System.currentTimeMillis();
+            log("To process the files took " + (endTime - startTime) + " ms.",
+                Project.MSG_VERBOSE);
+            final int numWarnings = warningCounter.getCount();
+            final boolean ok = (numErrs <= mMaxErrors)
+                    && (numWarnings <= mMaxWarnings);
 
             // Handle the return status
-            if ((numErrs > 0) && mFailureProperty != null) {
-                getProject().setProperty(mFailureProperty, "true");
-            }
+            if (!ok) {
+                final String failureMsg =
+                        "Got " + numErrs + " errors and " + numWarnings
+                                + " warnings.";
+                if (mFailureProperty != null) {
+                    getProject().setProperty(mFailureProperty, failureMsg);
+                }
 
-            if ((numErrs > 0) && mFailOnViolation) {
-                throw new BuildException("Got " + numErrs + " errors.",
-                                         getLocation());
+                if (mFailOnViolation) {
+                    throw new BuildException(failureMsg, getLocation());
+                }
             }
         }
         finally {
@@ -290,38 +355,38 @@ public class CheckStyleTask
             final Properties props = createOverridingProperties();
             final Configuration config =
                 ConfigurationLoader.loadConfiguration(
-                    mConfigLocation, new PropertiesExpander(props));
+                    mConfigLocation,
+                    new PropertiesExpander(props),
+                    mOmitIgnoredModules);
 
             final DefaultContext context = new DefaultContext();
-            final ClassLoader loader =
-                new AntClassLoader(getProject(), mClasspath);
+            final ClassLoader loader = new AntClassLoader(getProject(),
+                    mClasspath);
             context.add("classloader", loader);
 
+            final ClassLoader moduleClassLoader =
+                Checker.class.getClassLoader();
+            context.add("moduleClassLoader", moduleClassLoader);
+
             c = new Checker();
 
-            //load the set of package names
-            if (mPackageNamesFile != null) {
-                final ModuleFactory moduleFactory =
-                    PackageNamesLoader.loadModuleFactory(
-                        mPackageNamesFile.getAbsolutePath());
-                c.setModuleFactory(moduleFactory);
-            }
             c.contextualize(context);
             c.configure(config);
 
             // setup the listeners
             final AuditListener[] listeners = getListeners();
-            for (int i = 0; i < listeners.length; i++) {
-                c.addListener(listeners[i]);
+            for (AuditListener element : listeners) {
+                c.addListener(element);
             }
         }
-        catch (Exception e) {
-            throw new BuildException(
-                "Unable to create a Checker: " + e.getMessage(), e);
+        catch (final Exception e) {
+            throw new BuildException("Unable to create a Checker: "
+                    + e.getMessage(), e);
         }
 
         return c;
     }
+
     /**
      * Create the Properties object based on the arguments specified
      * to the ANT task.
@@ -339,42 +404,29 @@ public class CheckStyleTask
                 inStream = new FileInputStream(mPropertiesFile);
                 retVal.load(inStream);
             }
-            catch (FileNotFoundException e) {
-                throw new BuildException(
-                    "Could not find Properties file '" + mPropertiesFile + "'",
-                    e, getLocation());
+            catch (final FileNotFoundException e) {
+                throw new BuildException("Could not find Properties file '"
+                        + mPropertiesFile + "'", e, getLocation());
             }
-            catch (IOException e) {
-                throw new BuildException(
-                    "Error loading Properties file '" + mPropertiesFile + "'",
-                    e, getLocation());
+            catch (final IOException e) {
+                throw new BuildException("Error loading Properties file '"
+                        + mPropertiesFile + "'", e, getLocation());
             }
             finally {
-                try {
-                    if (inStream != null) {
-                        inStream.close();
-                    }
-                }
-                catch (IOException e) {
-                    throw new BuildException(
-                        "Error closing Properties file '"
-                        + mPropertiesFile + "'",
-                        e, getLocation());
-                }
+                Utils.closeQuietly(inStream);
             }
         }
 
         // override with Ant properties like ${basedir}
-        Hashtable antProps = this.getProject().getProperties();
-        for (Iterator it = antProps.keySet().iterator(); it.hasNext();) {
-            final String key = (String) it.next();
+        final Hashtable<?, ?> antProps = this.getProject().getProperties();
+        for (Object name : antProps.keySet()) {
+            final String key = (String) name;
             final String value = String.valueOf(antProps.get(key));
             retVal.put(key, value);
         }
 
         // override with properties specified in subelements
-        for (Iterator it = mOverrideProps.iterator(); it.hasNext();) {
-            final Property p = (Property) it.next();
+        for (Property p : mOverrideProps) {
             retVal.put(p.getKey(), p.getValue());
         }
 
@@ -389,24 +441,23 @@ public class CheckStyleTask
      * @throws IllegalAccessException if an error occurs
      * @throws IOException if an error occurs
      */
-    protected AuditListener[] getListeners()
-        throws ClassNotFoundException, InstantiationException,
-        IllegalAccessException, IOException
+    protected AuditListener[] getListeners() throws ClassNotFoundException,
+            InstantiationException, IllegalAccessException, IOException
     {
         final int formatterCount = Math.max(1, mFormatters.size());
 
-        final AuditListener[] listeners =
-            new AuditListener[formatterCount];
+        final AuditListener[] listeners = new AuditListener[formatterCount];
 
         // formatters
         if (mFormatters.size() == 0) {
-            OutputStream debug = new LogOutputStream(this, Project.MSG_DEBUG);
-            OutputStream err = new LogOutputStream(this, Project.MSG_ERR);
+            final OutputStream debug = new LogOutputStream(this,
+                    Project.MSG_DEBUG);
+            final OutputStream err = new LogOutputStream(this, Project.MSG_ERR);
             listeners[0] = new DefaultLogger(debug, true, err, true);
         }
         else {
             for (int i = 0; i < formatterCount; i++) {
-                final Formatter f = (Formatter) mFormatters.get(i);
+                final Formatter f = mFormatters.get(i);
                 listeners[i] = f.createListener(this);
             }
         }
@@ -417,9 +468,9 @@ public class CheckStyleTask
      * returns the list of files (full path name) to process.
      * @return the list of files included via the filesets.
      */
-    protected File[] scanFileSets()
+    protected List<File> scanFileSets()
     {
-        final ArrayList list = new ArrayList();
+        final List<File> list = Lists.newArrayList();
         if (mFileName != null) {
             // oops we've got an additional one to process, don't
             // forget it. No sweat, it's fully resolved via the setter.
@@ -427,39 +478,37 @@ public class CheckStyleTask
             list.add(new File(mFileName));
         }
         for (int i = 0; i < mFileSets.size(); i++) {
-            final FileSet fs = (FileSet) mFileSets.get(i);
+            final FileSet fs = mFileSets.get(i);
             final DirectoryScanner ds = fs.getDirectoryScanner(getProject());
             ds.scan();
 
             final String[] names = ds.getIncludedFiles();
             log(i + ") Adding " + names.length + " files from directory "
-                + ds.getBasedir(),
-                Project.MSG_VERBOSE);
+                    + ds.getBasedir(), Project.MSG_VERBOSE);
 
-            for (int j = 0; j < names.length; j++) {
-                final String pathname =
-                    ds.getBasedir() + File.separator + names[j];
+            for (String element : names) {
+                final String pathname = ds.getBasedir() + File.separator
+                        + element;
                 list.add(new File(pathname));
             }
         }
 
-        return (File[]) list.toArray(new File[0]);
+        return list;
     }
 
     /**
      * Poor mans enumeration for the formatter types.
      * @author Oliver Burn
      */
-    public static class FormatterType
-        extends EnumeratedAttribute
+    public static class FormatterType extends EnumeratedAttribute
     {
         /** my possible values */
         private static final String[] VALUES = {E_XML, E_PLAIN};
 
-        /** @see EnumeratedAttribute */
+        @Override
         public String[] getValues()
         {
-            return VALUES;
+            return VALUES.clone();
         }
     }
 
@@ -473,6 +522,8 @@ public class CheckStyleTask
         private FormatterType mFormatterType;
         /** the file to output to */
         private File mToFile;
+        /** Whether or not the write to the named file. */
+        private boolean mUseFile = true;
 
         /**
          * Set the type of the formatter.
@@ -497,23 +548,29 @@ public class CheckStyleTask
             mToFile = aTo;
         }
 
+        /**
+         * Sets whether or not we write to a file if it is provided.
+         * @param aUse whether not not to use provided file.
+         */
+        public void setUseFile(boolean aUse)
+        {
+            mUseFile = aUse;
+        }
+
         /**
          * Creates a listener for the formatter.
          * @param aTask the task running
          * @return a listener
          * @throws IOException if an error occurs
          */
-        public AuditListener createListener(Task aTask)
-            throws IOException
+        public AuditListener createListener(Task aTask) throws IOException
         {
             if ((mFormatterType != null)
-                && E_XML.equals(mFormatterType.getValue()))
+                    && E_XML.equals(mFormatterType.getValue()))
             {
                 return createXMLLogger(aTask);
             }
-            else {
-                return createDefaultLogger(aTask);
-            }
+            return createDefaultLogger(aTask);
         }
 
         /**
@@ -524,10 +581,10 @@ public class CheckStyleTask
         private AuditListener createDefaultLogger(Task aTask)
             throws IOException
         {
-            if (mToFile == null) {
+            if ((mToFile == null) || !mUseFile) {
                 return new DefaultLogger(
-                    new LogOutputStream(aTask, Project.MSG_DEBUG), true,
-                    new LogOutputStream(aTask, Project.MSG_ERR), true);
+                    new LogOutputStream(aTask, Project.MSG_DEBUG),
+                    true, new LogOutputStream(aTask, Project.MSG_ERR), true);
             }
             return new DefaultLogger(new FileOutputStream(mToFile), true);
         }
@@ -537,16 +594,13 @@ public class CheckStyleTask
          * @param aTask the task to possibly log to
          * @throws IOException if an error occurs
          */
-        private AuditListener createXMLLogger(Task aTask)
-            throws IOException
+        private AuditListener createXMLLogger(Task aTask) throws IOException
         {
-            if (mToFile == null) {
-                return new XMLLogger(
-                    new LogOutputStream(aTask, Project.MSG_INFO), true);
-            }
-            else {
-                return new XMLLogger(new FileOutputStream(mToFile), true);
+            if ((mToFile == null) || !mUseFile) {
+                return new XMLLogger(new LogOutputStream(aTask,
+                        Project.MSG_INFO), true);
             }
+            return new XMLLogger(new FileOutputStream(mToFile), true);
         }
     }
 
