@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,29 +18,31 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle;
 
-import java.io.BufferedInputStream;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.InputStream;
-import java.net.MalformedURLException;
-import java.net.URL;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Stack;
-import javax.xml.parsers.ParserConfigurationException;
-
+import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 import com.puppycrawl.tools.checkstyle.api.AbstractLoader;
 import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
 import com.puppycrawl.tools.checkstyle.api.Configuration;
+import com.puppycrawl.tools.checkstyle.api.FastStack;
+import com.puppycrawl.tools.checkstyle.api.SeverityLevel;
 import org.xml.sax.Attributes;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 
+import javax.xml.parsers.ParserConfigurationException;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.InputStream;
+import java.net.MalformedURLException;
+import java.net.URI;
+import java.net.URISyntaxException;
+import java.net.URL;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+
 /**
  * Loads a configuration from a standard configuration XML file.
  *
@@ -69,12 +71,17 @@ public final class ConfigurationLoader
     private static final String DTD_PUBLIC_ID_1_2 =
         "-//Puppy Crawl//DTD Check Configuration 1.2//EN";
 
-    /** the resource for version 1_1 of the configuration dtd */
+    /** the resource for version 1_2 of the configuration dtd */
     private static final String DTD_RESOURCE_NAME_1_2 =
         "com/puppycrawl/tools/checkstyle/configuration_1_2.dtd";
 
-    /** constant to specify two kilobyte of data */
-    private static final int TWO_KB = 2048;
+    /** the public ID for version 1_3 of the configuration dtd */
+    private static final String DTD_PUBLIC_ID_1_3 =
+        "-//Puppy Crawl//DTD Check Configuration 1.3//EN";
+
+    /** the resource for version 1_3 of the configuration dtd */
+    private static final String DTD_RESOURCE_NAME_1_3 =
+        "com/puppycrawl/tools/checkstyle/configuration_1_3.dtd";
 
     /**
      * Implements the SAX document handler interfaces, so they do not
@@ -93,6 +100,12 @@ public final class ConfigurationLoader
         private static final String VALUE = "value";
         /** default attribute */
         private static final String DEFAULT = "default";
+        /** name of the severity property */
+        private static final String SEVERITY = "severity";
+        /** name of the message element */
+        private static final String MESSAGE = "message";
+        /** name of the key attribute */
+        private static final String KEY = "key";
 
         /**
          * Creates a new InternalLoader.
@@ -102,18 +115,18 @@ public final class ConfigurationLoader
         private InternalLoader()
             throws SAXException, ParserConfigurationException
         {
-//            super(DTD_PUBLIC_ID_1_1, DTD_RESOURCE_NAME_1_1);
+            // super(DTD_PUBLIC_ID_1_1, DTD_RESOURCE_NAME_1_1);
             super(createIdToResourceNameMap());
         }
 
-        /** @see org.xml.sax.helpers.DefaultHandler **/
+        @Override
         public void startElement(String aNamespaceURI,
                                  String aLocalName,
                                  String aQName,
                                  Attributes aAtts)
             throws SAXException
         {
-            // TODO: debug logging for support puposes
+            // TODO: debug logging for support purposes
             if (aQName.equals(MODULE)) {
                 //create configuration
                 final String name = aAtts.getValue(NAME);
@@ -127,7 +140,7 @@ public final class ConfigurationLoader
                 //add configuration to it's parent
                 if (!mConfigStack.isEmpty()) {
                     final DefaultConfiguration top =
-                        (DefaultConfiguration) mConfigStack.peek();
+                        mConfigStack.peek();
                     top.addChild(conf);
                 }
 
@@ -141,85 +154,125 @@ public final class ConfigurationLoader
                     value = replaceProperties(aAtts.getValue(VALUE),
                         mOverridePropsResolver, aAtts.getValue(DEFAULT));
                 }
-                catch (CheckstyleException ex) {
+                catch (final CheckstyleException ex) {
                     throw new SAXException(ex.getMessage());
                 }
 
                 //add to attributes of configuration
                 final DefaultConfiguration top =
-                    (DefaultConfiguration) mConfigStack.peek();
+                    mConfigStack.peek();
                 top.addAttribute(name, value);
             }
+            else if (aQName.equals(MESSAGE)) {
+                //extract key and value
+                final String key = aAtts.getValue(KEY);
+                final String value = aAtts.getValue(VALUE);
+
+                //add to messages of configuration
+                final DefaultConfiguration top = mConfigStack.peek();
+                top.addMessage(key, value);
+            }
         }
 
-        /** @see org.xml.sax.helpers.DefaultHandler **/
+        @Override
         public void endElement(String aNamespaceURI,
                                String aLocalName,
                                String aQName)
+            throws SAXException
         {
             if (aQName.equals(MODULE)) {
-                mConfigStack.pop();
+
+                final Configuration recentModule =
+                    mConfigStack.pop();
+
+                // remove modules with severity ignore if these modules should
+                // be omitted
+                SeverityLevel level = null;
+                try {
+                    final String severity = recentModule.getAttribute(SEVERITY);
+                    level = SeverityLevel.getInstance(severity);
+                }
+                catch (final CheckstyleException e) {
+                    //severity not set -> ignore
+                    ;
+                }
+
+                // omit this module if these should be omitted and the module
+                // has the severity 'ignore'
+                final boolean omitModule = mOmitIgnoredModules
+                    && SeverityLevel.IGNORE.equals(level);
+
+                if (omitModule && !mConfigStack.isEmpty()) {
+                    final DefaultConfiguration parentModule =
+                        mConfigStack.peek();
+                    parentModule.removeChild(recentModule);
+                }
             }
         }
 
     }
 
     /** the SAX document handler */
-    private InternalLoader mSaxHandler;
+    private final InternalLoader mSaxHandler;
 
     /** property resolver **/
     private final PropertyResolver mOverridePropsResolver;
     /** the loaded configurations **/
-    private final Stack mConfigStack = new Stack();
+    private final FastStack<DefaultConfiguration> mConfigStack =
+        FastStack.newInstance();
     /** the Configuration that is being built */
     private Configuration mConfiguration;
 
+    /** flags if modules with the severity 'ignore' should be omitted. */
+    private final boolean mOmitIgnoredModules;
+
     /**
      * Creates mapping between local resources and dtd ids.
      * @return map between local resources and dtd ids.
      */
-    private static Map createIdToResourceNameMap()
+    private static Map<String, String> createIdToResourceNameMap()
     {
-        Map map = new HashMap();
+        final Map<String, String> map = Maps.newHashMap();
         map.put(DTD_PUBLIC_ID_1_0, DTD_RESOURCE_NAME_1_0);
         map.put(DTD_PUBLIC_ID_1_1, DTD_RESOURCE_NAME_1_1);
         map.put(DTD_PUBLIC_ID_1_2, DTD_RESOURCE_NAME_1_2);
+        map.put(DTD_PUBLIC_ID_1_3, DTD_RESOURCE_NAME_1_3);
         return map;
     }
 
-
     /**
      * Creates a new <code>ConfigurationLoader</code> instance.
      * @param aOverrideProps resolver for overriding properties
+     * @param aOmitIgnoredModules <code>true</code> if ignored modules should be
+     *         omitted
      * @throws ParserConfigurationException if an error occurs
      * @throws SAXException if an error occurs
      */
-    private ConfigurationLoader(PropertyResolver aOverrideProps)
+    private ConfigurationLoader(final PropertyResolver aOverrideProps,
+                                final boolean aOmitIgnoredModules)
         throws ParserConfigurationException, SAXException
     {
         mSaxHandler = new InternalLoader();
         mOverridePropsResolver = aOverrideProps;
+        mOmitIgnoredModules = aOmitIgnoredModules;
     }
 
     /**
-     * Parses the specified stream loading the configuration information.
-     * The stream is NOT closed after parsing, it is the responsibility of
+     * Parses the specified input source loading the configuration information.
+     * The stream wrapped inside the source, if any, is NOT
+     * explicitely closed after parsing, it is the responsibility of
      * the caller to close the stream.
      *
-     * @param aStream the stream that contains the configuration data
+     * @param aSource the source that contains the configuration data
      * @throws IOException if an error occurs
      * @throws SAXException if an error occurs
      */
-    private void parseInputStream(InputStream aStream)
+    private void parseInputSource(InputSource aSource)
         throws IOException, SAXException
     {
-        final InputStream configStream =
-            new BufferedInputStream(aStream, TWO_KB);
-        final InputSource inputSource = new InputSource(configStream);
-        mSaxHandler.parseInputSource(inputSource);
+        mSaxHandler.parseInputSource(aSource);
     }
 
-
     /**
      * Returns the module configurations in a specified file.
      * @param aConfig location of config file, can be either a URL or a filename
@@ -227,47 +280,138 @@ public final class ConfigurationLoader
      * @return the check configurations
      * @throws CheckstyleException if an error occurs
      */
-    public static Configuration loadConfiguration(
-        String aConfig, PropertyResolver aOverridePropsResolver)
+    public static Configuration loadConfiguration(String aConfig,
+            PropertyResolver aOverridePropsResolver) throws CheckstyleException
+    {
+        return loadConfiguration(aConfig, aOverridePropsResolver, false);
+    }
+
+    /**
+     * Returns the module configurations in a specified file.
+     *
+     * @param aConfig location of config file, can be either a URL or a filename
+     * @param aOverridePropsResolver overriding properties
+     * @param aOmitIgnoredModules <code>true</code> if modules with severity
+     *            'ignore' should be omitted, <code>false</code> otherwise
+     * @return the check configurations
+     * @throws CheckstyleException if an error occurs
+     */
+    public static Configuration loadConfiguration(String aConfig,
+        PropertyResolver aOverridePropsResolver, boolean aOmitIgnoredModules)
+        throws CheckstyleException
+    {
+        try {
+            // figure out if this is a File or a URL
+            URI uri;
+            try {
+                final URL url = new URL(aConfig);
+                uri = url.toURI();
+            }
+            catch (final MalformedURLException ex) {
+                uri = null;
+            }
+            catch (final URISyntaxException ex) {
+                // URL violating RFC 2396
+                uri = null;
+            }
+            if (uri == null) {
+                final File file = new File(aConfig);
+                if (file.exists()) {
+                    uri = file.toURI();
+                }
+                else {
+                    // check to see if the file is in the classpath
+                    try {
+                        final URL configUrl = ConfigurationLoader.class
+                                .getResource(aConfig);
+                        if (configUrl == null) {
+                            throw new FileNotFoundException(aConfig);
+                        }
+                        uri = configUrl.toURI();
+                    }
+                    catch (final URISyntaxException e) {
+                        throw new FileNotFoundException(aConfig);
+                    }
+                }
+            }
+            final InputSource source = new InputSource(uri.toString());
+            return loadConfiguration(source, aOverridePropsResolver,
+                    aOmitIgnoredModules);
+        }
+        catch (final FileNotFoundException e) {
+            throw new CheckstyleException("unable to find " + aConfig, e);
+        }
+        catch (final CheckstyleException e) {
+                //wrap again to add file name info
+            throw new CheckstyleException("unable to read " + aConfig + " - "
+                    + e.getMessage(), e);
+        }
+    }
+
+    /**
+     * Returns the module configurations from a specified input stream.
+     * Note that clients are required to close the given stream by themselves
+     *
+     * @param aConfigStream the input stream to the Checkstyle configuration
+     * @param aOverridePropsResolver overriding properties
+     * @param aOmitIgnoredModules <code>true</code> if modules with severity
+     *            'ignore' should be omitted, <code>false</code> otherwise
+     * @return the check configurations
+     * @throws CheckstyleException if an error occurs
+     *
+     * @deprecated As this method does not provide a valid system ID,
+     *   preventing resolution of external entities, a
+     *   {@link #loadConfiguration(InputSource,PropertyResolver,boolean)
+     *          version using an InputSource}
+     *   should be used instead
+     */
+    @Deprecated
+    public static Configuration loadConfiguration(InputStream aConfigStream,
+        PropertyResolver aOverridePropsResolver, boolean aOmitIgnoredModules)
+        throws CheckstyleException
+    {
+        return loadConfiguration(new InputSource(aConfigStream),
+                                 aOverridePropsResolver, aOmitIgnoredModules);
+    }
+
+    /**
+     * Returns the module configurations from a specified input source.
+     * Note that if the source does wrap an open byte or character
+     * stream, clients are required to close that stream by themselves
+     *
+     * @param aConfigSource the input stream to the Checkstyle configuration
+     * @param aOverridePropsResolver overriding properties
+     * @param aOmitIgnoredModules <code>true</code> if modules with severity
+     *            'ignore' should be omitted, <code>false</code> otherwise
+     * @return the check configurations
+     * @throws CheckstyleException if an error occurs
+     */
+    public static Configuration loadConfiguration(InputSource aConfigSource,
+        PropertyResolver aOverridePropsResolver, boolean aOmitIgnoredModules)
         throws CheckstyleException
     {
         try {
             final ConfigurationLoader loader =
-                new ConfigurationLoader(aOverridePropsResolver);
-            // figure out if this is a File or a URL
-            InputStream configStream;
-            try {
-                URL url = new URL(aConfig);
-                configStream = url.openStream();
-            }
-            catch (MalformedURLException ex) {
-                configStream = new FileInputStream(aConfig);
-            }
-            final InputStream bufferedStream =
-                new BufferedInputStream(configStream);
-            loader.parseInputStream(bufferedStream);
-            bufferedStream.close();
+                new ConfigurationLoader(aOverridePropsResolver,
+                                        aOmitIgnoredModules);
+            loader.parseInputSource(aConfigSource);
             return loader.getConfiguration();
         }
-        catch (FileNotFoundException e) {
+        catch (final ParserConfigurationException e) {
             throw new CheckstyleException(
-                "unable to find " + aConfig, e);
+                "unable to parse configuration stream", e);
         }
-        catch (ParserConfigurationException e) {
-            throw new CheckstyleException(
-                "unable to parse " + aConfig, e);
-        }
-        catch (SAXParseException e) {
-            throw new CheckstyleException("unable to parse "
-                    + aConfig + " - " + e.getMessage() + ":" + e.getLineNumber()
+        catch (final SAXParseException e) {
+            throw new CheckstyleException("unable to parse configuration stream"
+                    + " - " + e.getMessage() + ":" + e.getLineNumber()
                     + ":" + e.getColumnNumber(), e);
         }
-        catch (SAXException e) {
-            throw new CheckstyleException("unable to parse "
-                    + aConfig + " - " + e.getMessage(), e);
+        catch (final SAXException e) {
+            throw new CheckstyleException("unable to parse configuration stream"
+                    + " - " + e.getMessage(), e);
         }
-        catch (IOException e) {
-            throw new CheckstyleException("unable to read " + aConfig, e);
+        catch (final IOException e) {
+            throw new CheckstyleException("unable to read from stream", e);
         }
     }
 
@@ -312,17 +456,17 @@ public final class ConfigurationLoader
             return null;
         }
 
-        final List fragments = new ArrayList();
-        final List propertyRefs = new ArrayList();
+        final List<String> fragments = Lists.newArrayList();
+        final List<String> propertyRefs = Lists.newArrayList();
         parsePropertyString(aValue, fragments, propertyRefs);
 
         final StringBuffer sb = new StringBuffer();
-        final Iterator i = fragments.iterator();
-        final Iterator j = propertyRefs.iterator();
+        final Iterator<String> i = fragments.iterator();
+        final Iterator<String> j = propertyRefs.iterator();
         while (i.hasNext()) {
-            String fragment = (String) i.next();
+            String fragment = i.next();
             if (fragment == null) {
-                final String propertyName = (String) j.next();
+                final String propertyName = j.next();
                 fragment = aProps.resolve(propertyName);
                 if (fragment == null) {
                     if (aDefaultValue != null) {
@@ -358,8 +502,8 @@ public final class ConfigurationLoader
      * http://cvs.apache.org/viewcvs/jakarta-ant/src/main/org/apache/tools/ant/ProjectHelper.java
      */
     private static void parsePropertyString(String aValue,
-                                           List aFragments,
-                                           List aPropertyRefs)
+                                           List<String> aFragments,
+                                           List<String> aPropertyRefs)
         throws CheckstyleException
     {
         int prev = 0;
@@ -418,5 +562,4 @@ public final class ConfigurationLoader
             aFragments.add(aValue.substring(prev));
         }
     }
-
 }
