@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2010  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,7 @@ package com.puppycrawl.tools.checkstyle.api;
  *
  * @author o_sukhodolsky
  */
-class Comment implements TextBlock
+public class Comment implements TextBlock
 {
     /** text of the comment. */
     private final String[] mText;
