@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2010  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,13 +18,21 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle.checks.imports;
 
+import com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocTags;
+
 import com.google.common.collect.Sets;
 import com.puppycrawl.tools.checkstyle.api.Check;
 import com.puppycrawl.tools.checkstyle.api.DetailAST;
+import com.puppycrawl.tools.checkstyle.api.FileContents;
 import com.puppycrawl.tools.checkstyle.api.FullIdent;
+import com.puppycrawl.tools.checkstyle.api.TextBlock;
 import com.puppycrawl.tools.checkstyle.api.TokenTypes;
 import com.puppycrawl.tools.checkstyle.api.Utils;
+import com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocTag;
+import com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocUtils;
 import java.util.Set;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 /**
  * <p>
@@ -40,12 +48,13 @@ import java.util.Set;
  * Compatible with Java 1.5 source.
  *
  * @author Oliver Burn
- * @version 1.1
  */
 public class UnusedImportsCheck extends Check
 {
     /** flag to indicate when time to start collecting references */
     private boolean mCollect;
+    /** flag whether to process Javadoc comments. */
+    private boolean mProcessJavadoc;
 
     /** set of the imports */
     private final Set<FullIdent> mImports = Sets.newHashSet();
@@ -58,6 +67,11 @@ public class UnusedImportsCheck extends Check
     {
     }
 
+    public void setProcessJavadoc(boolean aValue)
+    {
+        mProcessJavadoc = aValue;
+    }
+
     @Override
     public void beginTree(DetailAST aRootAST)
     {
@@ -70,7 +84,7 @@ public class UnusedImportsCheck extends Check
     public void finishTree(DetailAST aRootAST)
     {
         // loop over all the imports to see if referenced.
-        for (FullIdent imp : mImports) {
+        for (final FullIdent imp : mImports) {
             if (!mReferenced.contains(Utils.baseClassname(imp.getText()))) {
                 log(imp.getLineNo(),
                     imp.getColumnNo(),
@@ -83,14 +97,20 @@ public class UnusedImportsCheck extends Check
     public int[] getDefaultTokens()
     {
         return new int[] {
-            TokenTypes.PACKAGE_DEF,
-            TokenTypes.ANNOTATION_DEF,
-            TokenTypes.CLASS_DEF,
-            TokenTypes.ENUM_DEF,
             TokenTypes.IDENT,
             TokenTypes.IMPORT,
-            TokenTypes.INTERFACE_DEF,
             TokenTypes.STATIC_IMPORT,
+            // Definitions that may contain Javadoc...
+            TokenTypes.PACKAGE_DEF,
+            TokenTypes.ANNOTATION_DEF,
+            TokenTypes.ANNOTATION_FIELD_DEF,
+            TokenTypes.ENUM_DEF,
+            TokenTypes.ENUM_CONSTANT_DEF,
+            TokenTypes.CLASS_DEF,
+            TokenTypes.INTERFACE_DEF,
+            TokenTypes.METHOD_DEF,
+            TokenTypes.CTOR_DEF,
+            TokenTypes.VARIABLE_DEF,
         };
     }
 
@@ -114,13 +134,11 @@ public class UnusedImportsCheck extends Check
         else if (aAST.getType() == TokenTypes.STATIC_IMPORT) {
             processStaticImport(aAST);
         }
-        else if ((aAST.getType() == TokenTypes.CLASS_DEF)
-            || (aAST.getType() == TokenTypes.INTERFACE_DEF)
-            || (aAST.getType() == TokenTypes.ENUM_DEF)
-            || (aAST.getType() == TokenTypes.ANNOTATION_DEF)
-            || (aAST.getType() == TokenTypes.PACKAGE_DEF))
-        {
+        else {
             mCollect = true;
+            if (mProcessJavadoc) {
+                processJavadoc(aAST);
+            }
         }
     }
 
@@ -166,4 +184,32 @@ public class UnusedImportsCheck extends Check
             mImports.add(name);
         }
     }
+
+    /**
+     * Collects references made in JavaDoc comments.
+     * @param aAST node to inspect for JavaDoc
+     */
+    private void processJavadoc(DetailAST aAST)
+    {
+        final FileContents contents = getFileContents();
+        final int lineNo = aAST.getLineNo();
+        final TextBlock cmt = contents.getJavadocBefore(lineNo);
+        if (cmt != null) {
+            final JavadocTags tags = JavadocUtils.getJavadocTags(cmt,
+                    JavadocUtils.JavadocTagType.ALL);
+            for (final JavadocTag tag : tags.getValidTags()) {
+                if (tag.canReferenceImports()) {
+                    String identifier = tag.getArg1();
+                    // Trim off method or link text
+                    final Pattern pattern =
+                        Utils.getPattern("(.+?)(?:\\s+|#|\\$).*");
+                    final Matcher matcher = pattern.matcher(identifier);
+                    if (matcher.find()) {
+                        identifier = matcher.group(1);
+                    }
+                    mReferenced.add(identifier);
+                }
+            }
+        }
+    }
 }
