@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,39 +18,27 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle.checks.javadoc;
 
-import java.util.Stack;
-
+import com.google.common.collect.ImmutableSortedSet;
 import com.puppycrawl.tools.checkstyle.api.Check;
 import com.puppycrawl.tools.checkstyle.api.DetailAST;
+import com.puppycrawl.tools.checkstyle.api.FastStack;
 import com.puppycrawl.tools.checkstyle.api.FileContents;
+import com.puppycrawl.tools.checkstyle.api.JavadocTagInfo;
 import com.puppycrawl.tools.checkstyle.api.Scope;
 import com.puppycrawl.tools.checkstyle.api.ScopeUtils;
 import com.puppycrawl.tools.checkstyle.api.TextBlock;
 import com.puppycrawl.tools.checkstyle.api.TokenTypes;
-import org.apache.regexp.RE;
-import org.apache.regexp.RESyntaxException;
+import com.puppycrawl.tools.checkstyle.checks.CheckUtils;
+import java.util.List;
+import java.util.Set;
+import java.util.regex.Pattern;
 
 /**
- * <p>Custom Checkstyle Check to validate Javadoc.
- * The following checks are performed:
- * <ul>
- * <li>Ensures the first sentence ends with proper punctuation (That is
- * a period, question mark, or exclaimation mark).  Javadoc  automatically
- * places the first sentence in the method summary table and index.  With out
- * proper punctuation the Javadoc may be malformed.
- * <li> Check text for Javadoc statements that do not have any description.
- * This includes both completely empty Javadoc, and Javadoc with only
- * tags such as @param and @return.
- * <li>Check text for incomplete html tags.  Verifies that HTML tags have
- * corresponding end tags and issues an UNCLOSED_HTML error if not.
- * An EXTRA_HTML error is issued if an end tag is found without a previous
- * open tag.
- * </ul>
- * <p>These checks were patterned after the checks made by the doclet
- * <code>com.sun.tools.doclets.doccheck.DocCheck</code>
+ * Custom Checkstyle Check to validate Javadoc.
  *
  * @author Chris Stillwell
  * @author Daniel Grenner
+ * @author Travis Schneeberger
  * @version 1.2
  */
 public class JavadocStyleCheck
@@ -63,8 +51,20 @@ public class JavadocStyleCheck
     private static final String EXTRA_HTML = "javadoc.extrahtml";
 
     /** HTML tags that do not require a close tag. */
-    private static final String[] SINGLE_TAG =
-    {"p", "br", "li", "dt", "dd", "td", "hr", "img", "tr", "th", "td"};
+    private static final Set<String> SINGLE_TAGS = ImmutableSortedSet.of("p",
+            "br", "li", "dt", "dd", "td", "hr", "img", "tr", "th", "td");
+
+    /** HTML tags that are allowed in java docs.
+     * From http://www.w3schools.com/tags/default.asp
+     * The froms and structure tags are not allowed
+     */
+    private static final Set<String> ALLOWED_TAGS = ImmutableSortedSet.of(
+            "a", "abbr", "acronym", "address", "area", "b", "bdo", "big",
+            "blockquote", "br", "caption", "cite", "code", "colgroup", "del",
+            "div", "dfn", "dl", "em", "fieldset", "h1", "h2", "h3", "h4", "h5",
+            "h6", "hr", "i", "img", "ins", "kbd", "li", "ol", "p", "pre", "q",
+            "samp", "small", "span", "strong", "style", "sub", "sup", "table",
+            "tbody", "td", "tfoot", "th", "thead", "tr", "tt", "ul");
 
     /** The scope to check. */
     private Scope mScope = Scope.PRIVATE;
@@ -72,8 +72,11 @@ public class JavadocStyleCheck
     /** the visibility scope where Javadoc comments shouldn't be checked **/
     private Scope mExcludeScope;
 
+    /** Format for matching the end of a sentence. */
+    private String mEndOfSentenceFormat = "([.?!][ \t\n\r\f<])|([.?!]$)";
+
     /** Regular expression for matching the end of a sentence. */
-    private RE mEndOfSentenceRE;
+    private Pattern mEndOfSentencePattern;
 
     /**
      * Indicates if the first sentence should be checked for proper end of
@@ -91,33 +94,35 @@ public class JavadocStyleCheck
      */
     private boolean mCheckEmptyJavadoc;
 
-    /**
-     * The default tokens this Check is used for.
-     * @see com.puppycrawl.tools.checkstyle.api.Check#getDefaultTokens()
-     */
+    @Override
     public int[] getDefaultTokens()
     {
         return new int[] {
             TokenTypes.INTERFACE_DEF,
             TokenTypes.CLASS_DEF,
+            TokenTypes.ANNOTATION_DEF,
+            TokenTypes.ENUM_DEF,
             TokenTypes.METHOD_DEF,
             TokenTypes.CTOR_DEF,
             TokenTypes.VARIABLE_DEF,
+            TokenTypes.ENUM_CONSTANT_DEF,
+            TokenTypes.ANNOTATION_FIELD_DEF,
+            TokenTypes.PACKAGE_DEF,
         };
     }
 
-    /**
-     * Called to process a token.
-     * @see com.puppycrawl.tools.checkstyle.api.Check
-     */
+    @Override
     public void visitToken(DetailAST aAST)
     {
         if (shouldCheck(aAST)) {
             final FileContents contents = getFileContents();
+            // Need to start searching for the comment before the annotations
+            // that may exist. Even if annotations are not defined on the
+            // package, the ANNOTATIONS AST is defined.
             final TextBlock cmt =
-                contents.getJavadocBefore(aAST.getLineNo());
+                contents.getJavadocBefore(aAST.getFirstChild().getLineNo());
 
-            checkComment(cmt);
+            checkComment(aAST, cmt);
         }
     }
 
@@ -128,44 +133,64 @@ public class JavadocStyleCheck
      */
     private boolean shouldCheck(final DetailAST aAST)
     {
+        if (aAST.getType() == TokenTypes.PACKAGE_DEF) {
+            return getFileContents().inPackageInfo();
+        }
+
         if (ScopeUtils.inCodeBlock(aAST)) {
             return false;
         }
 
-        final DetailAST mods = aAST.findFirstToken(TokenTypes.MODIFIERS);
-        final Scope declaredScope = ScopeUtils.getScopeFromMods(mods);
+        final Scope declaredScope;
+        if (aAST.getType() == TokenTypes.ENUM_CONSTANT_DEF) {
+            declaredScope = Scope.PUBLIC;
+        }
+        else {
+            declaredScope = ScopeUtils.getScopeFromMods(
+                aAST.findFirstToken(TokenTypes.MODIFIERS));
+        }
+
         final Scope scope =
-            ScopeUtils.inInterfaceBlock(aAST) ? Scope.PUBLIC : declaredScope;
+            ScopeUtils.inInterfaceOrAnnotationBlock(aAST)
+            ? Scope.PUBLIC : declaredScope;
         final Scope surroundingScope = ScopeUtils.getSurroundingScope(aAST);
 
         return scope.isIn(mScope)
             && ((surroundingScope == null) || surroundingScope.isIn(mScope))
             && ((mExcludeScope == null)
                 || !scope.isIn(mExcludeScope)
-                || (surroundingScope != null)
-                && !surroundingScope.isIn(mExcludeScope));
+                || ((surroundingScope != null)
+                && !surroundingScope.isIn(mExcludeScope)));
     }
 
     /**
      * Performs the various checks agains the Javadoc comment.
      *
+     * @param aAST the AST of the element being documented
      * @param aComment the source lines that make up the Javadoc comment.
      *
      * @see #checkFirstSentence(TextBlock)
-     * @see #checkHtml(TextBlock)
+     * @see #checkHtml(DetailAST, TextBlock)
      */
-    private void checkComment(TextBlock aComment)
+    private void checkComment(final DetailAST aAST, final TextBlock aComment)
     {
         if (aComment == null) {
+            /*checking for missing docs in JavadocStyleCheck is not consistent
+            with the rest of CheckStyle...  Even though, I didn't think it
+            made sense to make another csheck just to ensure that the
+            package-info.java file actually contains package Javadocs.*/
+            if (getFileContents().inPackageInfo()) {
+                log(aAST.getLineNo(), "javadoc.missing");
+            }
             return;
         }
 
         if (mCheckFirstSentence) {
-            checkFirstSentence(aComment);
+            checkFirstSentence(aAST, aComment);
         }
 
         if (mCheckHtml) {
-            checkHtml(aComment);
+            checkHtml(aAST, aComment);
         }
 
         if (mCheckEmptyJavadoc) {
@@ -174,21 +199,23 @@ public class JavadocStyleCheck
     }
 
     /**
-     * Checks that the first sentence ends with proper puctuation.  This method
+     * Checks that the first sentence ends with proper punctuation.  This method
      * uses a regular expression that checks for the presence of a period,
-     * question mark, or exclaimation mark followed either by whitespace, an
-     * HTML element, or the end of string. This method ignores {@inheritDoc}
-     * comments.
+     * question mark, or exclamation mark followed either by whitespace, an
+     * HTML element, or the end of string. This method ignores {_AT_inheritDoc}
+     * comments for TokenTypes that are valid for {_AT_inheritDoc}.
      *
+     * @param aAST the current node
      * @param aComment the source lines that make up the Javadoc comment.
      */
-    private void checkFirstSentence(TextBlock aComment)
+    private void checkFirstSentence(final DetailAST aAST, TextBlock aComment)
     {
         final String commentText = getCommentText(aComment.getText());
 
         if ((commentText.length() != 0)
-            && !getEndOfSentenceRE().match(commentText)
-            && !"{@inheritDoc}".equals(commentText))
+            && !getEndOfSentencePattern().matcher(commentText).find()
+            && !("{@inheritDoc}".equals(commentText)
+            && JavadocTagInfo.INHERIT_DOC.isValidOn(aAST)))
         {
             log(aComment.getStartLineNo(), "javadoc.noperiod");
         }
@@ -216,31 +243,17 @@ public class JavadocStyleCheck
     private String getCommentText(String[] aComments)
     {
         final StringBuffer buffer = new StringBuffer();
-        boolean foundTag = false;
-
-        for (int i = 0; i < aComments.length; i++) {
-            String line = aComments[i];
+        for (final String line : aComments) {
             final int textStart = findTextStart(line);
 
             if (textStart != -1) {
-                // Look for Javadoc tag that's not an inline tag
-                // it can appear within the comment text.
-                // Inline tags are denoted by curly braces: {@tag}
-                final int ndx = line.indexOf('@');
-                if ((ndx != -1)
-                    && (ndx == 0 || line.charAt(ndx - 1) != '{'))
-                {
-                    foundTag = true;
-                    line = line.substring(0, ndx);
+                if (line.charAt(textStart) == '@') {
+                    //we have found the tag section
+                    break;
                 }
-
                 buffer.append(line.substring(textStart));
                 trimTail(buffer);
                 buffer.append('\n');
-
-                if (foundTag) {
-                    break;
-                }
             }
         }
 
@@ -292,6 +305,10 @@ public class JavadocStyleCheck
                 aBuffer.deleteCharAt(i);
                 aBuffer.deleteCharAt(i - 1);
                 i--;
+                while (aBuffer.charAt(i - 1) == '*') {
+                    aBuffer.deleteCharAt(i - 1);
+                    i--;
+                }
             }
             else {
                 break;
@@ -301,17 +318,20 @@ public class JavadocStyleCheck
 
     /**
      * Checks the comment for HTML tags that do not have a corresponding close
-     * tag or a close tage that has no previous open tag.  This code was
+     * tag or a close tag that has no previous open tag.  This code was
      * primarily copied from the DocCheck checkHtml method.
      *
+     * @param aAST the node with the Javadoc
      * @param aComment the <code>TextBlock</code> which represents
      *                 the Javadoc comment.
      */
-    private void checkHtml(TextBlock aComment)
+    private void checkHtml(final DetailAST aAST, final TextBlock aComment)
     {
         final int lineno = aComment.getStartLineNo();
-        final Stack htmlStack = new Stack();
+        final FastStack<HtmlTag> htmlStack = FastStack.newInstance();
         final String[] text = aComment.getText();
+        final List<String> typeParameters =
+            CheckUtils.getTypeParameterNames(aAST);
 
         TagParser parser = null;
         parser = new TagParser(text, lineno);
@@ -321,7 +341,7 @@ public class JavadocStyleCheck
 
             if (tag.isIncompleteTag()) {
                 log(tag.getLineno(), "javadoc.incompleteTag",
-                    new Object[] {text[tag.getLineno() - lineno]});
+                    text[tag.getLineno() - lineno]);
                 return;
             }
             if (tag.isClosedTag()) {
@@ -329,7 +349,10 @@ public class JavadocStyleCheck
                 continue;
             }
             if (!tag.isCloseTag()) {
-                htmlStack.push(tag);
+                //We only push html tags that are allowed
+                if (isAllowedTag(tag)) {
+                    htmlStack.push(tag);
+                }
             }
             else {
                 // We have found a close tag.
@@ -350,9 +373,11 @@ public class JavadocStyleCheck
 
         // Identify any tags left on the stack.
         String lastFound = ""; // Skip multiples, like <b>...<b>
-        for (int i = 0; i < htmlStack.size(); i++) {
-            final HtmlTag htag = (HtmlTag) htmlStack.elementAt(i);
-            if (!isSingleTag(htag) && !htag.getId().equals(lastFound)) {
+        for (final HtmlTag htag : htmlStack) {
+            if (!isSingleTag(htag)
+                && !htag.getId().equals(lastFound)
+                && !typeParameters.contains(htag.getId()))
+            {
                 log(htag.getLineno(), htag.getPosition(), UNCLOSED_HTML, htag);
                 lastFound = htag.getId();
             }
@@ -368,26 +393,26 @@ public class JavadocStyleCheck
      * @param aHtmlStack the stack of opened HTML tags.
      * @param aToken the current HTML tag name that has been closed.
      */
-    private void checkUnclosedTags(Stack aHtmlStack, String aToken)
+    private void checkUnclosedTags(FastStack<HtmlTag> aHtmlStack, String aToken)
     {
-        final Stack unclosedTags = new Stack();
-        HtmlTag lastOpenTag = (HtmlTag) aHtmlStack.pop();
+        final FastStack<HtmlTag> unclosedTags = FastStack.newInstance();
+        HtmlTag lastOpenTag = aHtmlStack.pop();
         while (!aToken.equalsIgnoreCase(lastOpenTag.getId())) {
             // Find unclosed elements. Put them on a stack so the
             // output order won't be back-to-front.
             if (isSingleTag(lastOpenTag)) {
-                lastOpenTag = (HtmlTag) aHtmlStack.pop();
+                lastOpenTag = aHtmlStack.pop();
             }
             else {
                 unclosedTags.push(lastOpenTag);
-                lastOpenTag = (HtmlTag) aHtmlStack.pop();
+                lastOpenTag = aHtmlStack.pop();
             }
         }
 
         // Output the unterminated tags, if any
         String lastFound = ""; // Skip multiples, like <b>..<b>
-        for (int i = 0; i < unclosedTags.size(); i++) {
-            lastOpenTag = (HtmlTag) unclosedTags.get(i);
+        for (final HtmlTag htag : unclosedTags) {
+            lastOpenTag = htag;
             if (lastOpenTag.getId().equals(lastFound)) {
                 continue;
             }
@@ -407,17 +432,22 @@ public class JavadocStyleCheck
      */
     private boolean isSingleTag(HtmlTag aTag)
     {
-        boolean isSingleTag = false;
-        for (int i = 0; i < SINGLE_TAG.length; i++) {
-            // If its a singleton tag (<p>, <br>, etc.), ignore it
-            // Can't simply not put them on the stack, since singletons
-            // like <dt> and <dd> (unhappily) may either be terminated
-            // or not terminated. Both options are legal.
-            if (aTag.getId().equalsIgnoreCase(SINGLE_TAG[i])) {
-                isSingleTag = true;
-            }
-        }
-        return isSingleTag;
+        // If its a singleton tag (<p>, <br>, etc.), ignore it
+        // Can't simply not put them on the stack, since singletons
+        // like <dt> and <dd> (unhappily) may either be terminated
+        // or not terminated. Both options are legal.
+        return SINGLE_TAGS.contains(aTag.getId().toLowerCase());
+    }
+
+    /**
+     * Determines if the HtmlTag is one which is allowed in a javadoc.
+     *
+     * @param aTag the HtmlTag to check.
+     * @return <code>true</code> if the HtmlTag is an allowed html tag.
+     */
+    private boolean isAllowedTag(HtmlTag aTag)
+    {
+        return ALLOWED_TAGS.contains(aTag.getId().toLowerCase());
     }
 
     /**
@@ -429,15 +459,14 @@ public class JavadocStyleCheck
      * @return <code>false</code> if a previous open tag was found
      *         for the token.
      */
-    private boolean isExtraHtml(String aToken, Stack aHtmlStack)
+    private boolean isExtraHtml(String aToken, FastStack<HtmlTag> aHtmlStack)
     {
         boolean isExtra = true;
-        for (int i = 0; i < aHtmlStack.size(); i++) {
+        for (final HtmlTag td : aHtmlStack) {
             // Loop, looking for tags that are closed.
             // The loop is needed in case there are unclosed
             // tags on the stack. In that case, the stack would
             // not be empty, but this tag would still be extra.
-            HtmlTag td = (HtmlTag) aHtmlStack.elementAt(i);
             if (aToken.equalsIgnoreCase(td.getId())) {
                 isExtra = false;
                 break;
@@ -465,23 +494,26 @@ public class JavadocStyleCheck
         mExcludeScope = Scope.getInstance(aScope);
     }
 
+    /**
+     * Set the format for matching the end of a sentence.
+     * @param aFormat format for matching the end of a sentence.
+     */
+    public void setEndOfSentenceFormat(String aFormat)
+    {
+        mEndOfSentenceFormat = aFormat;
+    }
+
     /**
      * Returns a regular expression for matching the end of a sentence.
      *
      * @return a regular expression for matching the end of a sentence.
      */
-    private RE getEndOfSentenceRE()
+    private Pattern getEndOfSentencePattern()
     {
-        if (mEndOfSentenceRE == null) {
-            try {
-                mEndOfSentenceRE = new RE("([.?!][ \t\n\r\f<])|([.?!]$)");
-            }
-            catch (RESyntaxException e) {
-                // This should never occur.
-                e.printStackTrace();
-            }
+        if (mEndOfSentencePattern == null) {
+            mEndOfSentencePattern = Pattern.compile(mEndOfSentenceFormat);
         }
-        return mEndOfSentenceRE;
+        return mEndOfSentencePattern;
     }
 
     /**
