@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2010  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle.checks.javadoc;
 
-import com.google.common.collect.Lists;
 import com.puppycrawl.tools.checkstyle.api.Check;
 import com.puppycrawl.tools.checkstyle.api.DetailAST;
 import com.puppycrawl.tools.checkstyle.api.FileContents;
@@ -40,7 +39,6 @@ import org.apache.commons.beanutils.ConversionException;
  *
  * @author Oliver Burn
  * @author Michael Tamm
- * @version 1.1
  */
 public class JavadocTypeCheck
     extends Check
@@ -172,7 +170,7 @@ public class JavadocTypeCheck
 
                 if (!mAllowMissingParamTags) {
                     //Check type parameters that should exist, do
-                    for (String string : typeParamNames) {
+                    for (final String string : typeParamNames) {
                         checkTypeParamTag(
                             lineNo, tags, string);
                     }
@@ -207,41 +205,20 @@ public class JavadocTypeCheck
 
     /**
      * Gets all standalone tags from a given javadoc.
-     * @param aCmt teh Javadoc comment to process.
+     * @param aCmt the Javadoc comment to process.
      * @return all standalone tags from the given javadoc.
      */
     private List<JavadocTag> getJavadocTags(TextBlock aCmt)
     {
-        final String[] text = aCmt.getText();
-        final List<JavadocTag> tags = Lists.newArrayList();
-        Pattern tagPattern = Utils.getPattern("/\\*{2,}\\s*@(\\p{Alpha}+)\\s");
-        for (int i = 0; i < text.length; i++) {
-            final String s = text[i];
-            final Matcher tagMatcher = tagPattern.matcher(s);
-            if (tagMatcher.find()) {
-                final String tagName = tagMatcher.group(1);
-
-                String content = s.substring(tagMatcher.end(1));
-                if (content.endsWith("*/")) {
-                    content = content.substring(0, content.length() - 2);
-                }
-                int col = tagMatcher.start(1) - 1;
-                if (i == 0) {
-                    col += aCmt.getStartColNo();
-                }
-
-                if (JavadocTagInfo.isValidName(tagName)) {
-                    tags.add(new JavadocTag(aCmt.getStartLineNo() + i, col,
-                            tagName, content.trim()));
-                }
-                else if (!mAllowUnknownTags) {
-                    log(aCmt.getStartLineNo() + i, col,
-                            "javadoc.unknownTag", tagName);
-                }
+        final JavadocTags tags = JavadocUtils.getJavadocTags(aCmt,
+            JavadocUtils.JavadocTagType.BLOCK);
+        if (!mAllowUnknownTags) {
+            for (final InvalidJavadocTag tag : tags.getInvalidTags()) {
+                log(tag.getLine(), tag.getCol(), "javadoc.unknownTag",
+                    tag.getName());
             }
-            tagPattern = Utils.getPattern("^\\s*\\**\\s*@(\\p{Alpha}+)\\s");
         }
-        return tags;
+        return tags.getValidTags();
     }
 
     /**
