@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2004  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,40 +18,16 @@
 ////////////////////////////////////////////////////////////////////////////////
 package com.puppycrawl.tools.checkstyle.checks.whitespace;
 
-import com.puppycrawl.tools.checkstyle.checks.AbstractOption;
-import java.util.HashMap;
-import java.util.Map;
-
 /**
  * Represents the options for wrapping on an operator.
  *
  * @author Rick Giles
  * @version 1.0
  */
-public final class OperatorWrapOption
-    extends AbstractOption
+public enum OperatorWrapOption
 {
-    /** maps from a string representation to an option */
-    private static final Map STR_TO_OPT = new HashMap();
-
     /** Require that the operator is on a new line. */
-    public static final OperatorWrapOption NL = new OperatorWrapOption("nl");
+    NL,
     /** Require that the operator is at the end of the line. */
-    public static final OperatorWrapOption EOL = new OperatorWrapOption("eol");
-
-    /**
-     * Creates a new <code>OperatorWrapOption</code> instance.
-     *
-     * @param aStrRep the string representation
-     */
-    private OperatorWrapOption(String aStrRep)
-    {
-        super(aStrRep);
-    }
-
-    /** @see com.puppycrawl.tools.checkstyle.checks.AbstractOption */
-    protected Map getStrToOpt()
-    {
-        return STR_TO_OPT;
-    }
+    EOL;
 }
