@@ -18,17 +18,43 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- * @(#)JTreeTable.java	1.2 98/10/27
+ * %W% %E%
  *
- * Copyright 1997, 1998 by Sun Microsystems, Inc.,
- * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
- * All rights reserved.
+ * Copyright 1997, 1998 Sun Microsystems, Inc. All Rights Reserved.
  *
- * This software is the confidential and proprietary information
- * of Sun Microsystems, Inc. ("Confidential Information").  You
- * shall not disclose such Confidential Information and shall use
- * it only in accordance with the terms of the license agreement
- * you entered into with Sun.
+ * Redistribution and use in source and binary forms, with or
+ * without modification, are permitted provided that the following
+ * conditions are met:
+ *
+ * - Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+ *
+ * - Redistribution in binary form must reproduce the above
+ *   copyright notice, this list of conditions and the following
+ *   disclaimer in the documentation and/or other materials
+ *   provided with the distribution.
+ *
+ * Neither the name of Sun Microsystems, Inc. or the names of
+ * contributors may be used to endorse or promote products derived
+ * from this software without specific prior written permission.
+ *
+ * This software is provided "AS IS," without a warranty of any
+ * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
+ * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
+ * EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
+ * DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
+ * RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
+ * ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
+ * FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
+ * SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
+ * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
+ * THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
+ * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
+ *
+ * You acknowledge that this software is not designed, licensed or
+ * intended for use in the design, construction, operation or
+ * maintenance of any nuclear facility.
  */
 
 package com.puppycrawl.tools.checkstyle.gui;
@@ -62,7 +88,7 @@ import javax.swing.tree.TreePath;
  * by using a JTree as a renderer (and editor) for the cells in a
  * particular column in the JTable.
  *
- * @version 1.2 10/27/98
+ * <a href="http://java.sun.com/products/jfc/tsc/articles/treetable1/index.html">Original&nbsp;Source&nbsp;Location</a>
  *
  * @author Philip Milne
  * @author Scott Violet
@@ -70,6 +96,8 @@ import javax.swing.tree.TreePath;
  */
 public class JTreeTable extends JTable
 {
+    /** For Serialisation that will never happen. */
+    private static final long serialVersionUID = -8493693409423365387L;
     /** A subclass of JTree. */
     protected TreeTableCellRenderer tree;
 
@@ -84,7 +112,7 @@ public class JTreeTable extends JTable
         super.setModel(new TreeTableModelAdapter(treeTableModel, tree));
 
         // Force the JTable and JTree to share their row selection models.
-        ListToTreeSelectionModelWrapper selectionWrapper = new
+        final ListToTreeSelectionModelWrapper selectionWrapper = new
                 ListToTreeSelectionModelWrapper();
         tree.setSelectionModel(selectionWrapper);
         setSelectionModel(selectionWrapper.getListSelectionModel());
@@ -106,9 +134,14 @@ public class JTreeTable extends JTable
             setRowHeight(getRowHeight());
         }
 
-        Action expand = new AbstractAction() {
+        final Action expand = new AbstractAction() {
+                /**
+             *
+             */
+            private static final long serialVersionUID = -5859674518660156121L;
+
                 public void actionPerformed(ActionEvent e) {
-                    TreePath selected = tree.getSelectionPath();
+                    final TreePath selected = tree.getSelectionPath();
                     if (tree.isExpanded(selected)) {
                         tree.collapsePath(selected);
                     }
@@ -118,8 +151,8 @@ public class JTreeTable extends JTable
                     tree.setSelectionPath(selected);
                 }
             };
-        KeyStroke stroke = KeyStroke.getKeyStroke("ENTER");
-        String command = "expand/collapse";
+        final KeyStroke stroke = KeyStroke.getKeyStroke("ENTER");
+        final String command = "expand/collapse";
         getInputMap().put(stroke, command);
         getActionMap().put(command, expand);
     }
@@ -129,6 +162,7 @@ public class JTreeTable extends JTable
      * Since the tree is not actually in the component hierarchy it will
      * never receive this unless we forward it in this manner.
      */
+    @Override
     public void updateUI()
     {
         super.updateUI();
@@ -147,19 +181,21 @@ public class JTreeTable extends JTable
      * is not the right thing to do for an editor. Returning -1 for the
      * editing row in this case, ensures the editor is never painted.
      */
+    @Override
     public int getEditingRow()
     {
-        final Class editingClass = getColumnClass(editingColumn);
+        final Class<?> editingClass = getColumnClass(editingColumn);
         return (editingClass == TreeTableModel.class) ? -1 : editingRow;
     }
 
     /**
      * Overridden to pass the new rowHeight to the tree.
      */
-    public void setRowHeight(int rowHeight)
+    @Override
+    public void setRowHeight(int newRowHeight)
     {
-        super.setRowHeight(rowHeight);
-        if (tree != null && tree.getRowHeight() != rowHeight) {
+        super.setRowHeight(newRowHeight);
+        if ((tree != null) && (tree.getRowHeight() != newRowHeight)) {
             tree.setRowHeight(getRowHeight());
         }
     }
@@ -178,6 +214,10 @@ public class JTreeTable extends JTable
     class TreeTableCellRenderer extends JTree implements
             TableCellRenderer
     {
+        /**
+         *
+         */
+        private static final long serialVersionUID = 4324031590789321581L;
         /** Last table/tree row asked to renderer. */
         protected int visibleRow;
 
@@ -191,14 +231,15 @@ public class JTreeTable extends JTable
          * updateUI is overridden to set the colors of the Tree's renderer
          * to match that of the table.
          */
+        @Override
         public void updateUI()
         {
             super.updateUI();
             // Make the tree's cell renderer use the table's cell selection
             // colors.
-            TreeCellRenderer tcr = getCellRenderer();
+            final TreeCellRenderer tcr = getCellRenderer();
             if (tcr instanceof DefaultTreeCellRenderer) {
-                DefaultTreeCellRenderer dtcr = ((DefaultTreeCellRenderer) tcr);
+                final DefaultTreeCellRenderer dtcr = ((DefaultTreeCellRenderer) tcr);
                 // For 1.1 uncomment this, 1.2 has a bug that will cause an
                 // exception to be thrown if the border selection color is
                 // null.
@@ -214,12 +255,14 @@ public class JTreeTable extends JTable
          * Sets the row height of the tree, and forwards the row height to
          * the table.
          */
-        public void setRowHeight(int rowHeight)
+        @Override
+        public void setRowHeight(int newRowHeight)
         {
-            if (rowHeight > 0) {
-                super.setRowHeight(rowHeight);
-                if (JTreeTable.this != null &&
-                        JTreeTable.this.getRowHeight() != rowHeight) {
+            if (newRowHeight > 0) {
+                super.setRowHeight(newRowHeight);
+                if ((JTreeTable.this != null) &&
+                    (JTreeTable.this.getRowHeight() != newRowHeight))
+                {
                     JTreeTable.this.setRowHeight(getRowHeight());
                 }
             }
@@ -228,6 +271,7 @@ public class JTreeTable extends JTable
         /**
          * This is overridden to set the height to match that of the JTable.
          */
+        @Override
         public void setBounds(int x, int y, int w, int h)
         {
             super.setBounds(x, 0, w, JTreeTable.this.getHeight());
@@ -237,6 +281,7 @@ public class JTreeTable extends JTable
          * Sublcassed to translate the graphics such that the last visible
          * row will be drawn at 0,0.
          */
+        @Override
         public void paint(Graphics g)
         {
             g.translate(0, -visibleRow * getRowHeight());
@@ -300,14 +345,15 @@ public class JTreeTable extends JTable
          *
          * @see TableCellEditor
          */
+        @Override
         public boolean isCellEditable(EventObject e)
         {
             if (e instanceof MouseEvent) {
                 for (int counter = getColumnCount() - 1; counter >= 0;
                      counter--) {
                     if (getColumnClass(counter) == TreeTableModel.class) {
-                        MouseEvent me = (MouseEvent) e;
-                        MouseEvent newME = new MouseEvent(tree, me.getID(),
+                        final MouseEvent me = (MouseEvent) e;
+                        final MouseEvent newME = new MouseEvent(tree, me.getID(),
                                 me.getWhen(), me.getModifiers(),
                                 me.getX() - getCellRect(0, counter, true).x,
                                 me.getY(), me.getClickCount(),
@@ -331,6 +377,10 @@ public class JTreeTable extends JTable
      */
     class ListToTreeSelectionModelWrapper extends DefaultTreeSelectionModel
     {
+        /**
+         *
+         */
+        private static final long serialVersionUID = 2267930983939339510L;
         /** Set to true when we are updating the ListSelectionModel. */
         protected boolean updatingListSelectionModel;
 
@@ -358,6 +408,7 @@ public class JTreeTable extends JTable
          * and message super. This is the only place DefaultTreeSelectionModel
          * alters the ListSelectionModel.
          */
+        @Override
         public void resetRowSelection()
         {
             if (!updatingListSelectionModel) {
@@ -395,14 +446,14 @@ public class JTreeTable extends JTable
                 try {
                     // This is way expensive, ListSelectionModel needs an
                     // enumerator for iterating.
-                    int min = listSelectionModel.getMinSelectionIndex();
-                    int max = listSelectionModel.getMaxSelectionIndex();
+                    final int min = listSelectionModel.getMinSelectionIndex();
+                    final int max = listSelectionModel.getMaxSelectionIndex();
 
                     clearSelection();
-                    if (min != -1 && max != -1) {
+                    if ((min != -1) && (max != -1)) {
                         for (int counter = min; counter <= max; counter++) {
                             if (listSelectionModel.isSelectedIndex(counter)) {
-                                TreePath selPath = tree.getPathForRow
+                                final TreePath selPath = tree.getPathForRow
                                         (counter);
 
                                 if (selPath != null) {
