@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // checkstyle: Checks Java source code for adherence to a set of rules.
-// Copyright (C) 2001-2010  Oliver Burn
+// Copyright (C) 2001-2011  Oliver Burn
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,17 @@ import org.junit.Test;
  */
 public class SuppressionsLoaderTest
 {
+    @Test
+    public void testResourceLoadSuppressions()
+        throws CheckstyleException
+    {
+        final FilterSet fc =
+            SuppressionsLoader.loadSuppressions(
+                "/checkstyle/suppressions.xml");
+        final FilterSet fc2 = new FilterSet();
+        assertEquals(fc, fc2);
+    }
+
     @Test
     public void testNoSuppressions()
         throws CheckstyleException
